@@ -1,5 +1,6 @@
+"""Initializes a train object in api."""
 from ncem.estimators import (Estimator, EstimatorCVAE, EstimatorCVAEncem,
-                             EstimatorED, EstimatorEDncem, EstimatorGraph,
+                             EstimatorED, EstimatorEDncem, EstimatorEdNcemNeighborhood, EstimatorGraph,
                              EstimatorInteractions, EstimatorLinear,
                              EstimatorNoGraph)
 from ncem.models import BetaScheduler
