@@ -22,16 +22,17 @@ from ncem.utils.wald_test import wald_test
 
 
 class InterpreterBase(estimators.Estimator):
+    """InterpreterBase class."""
+
     data_path: str
     results_path: str
-
     model_class: str
     data_set: str
     radius: int
-
     cell_names: list
 
     def __init__(self):
+        """Initialize InterpreterBase."""
         super().__init__()
         self.cell_type = None
         self.position_matrix = None
@@ -47,6 +48,11 @@ class InterpreterBase(estimators.Estimator):
         self.reinit_model = None
 
     def init_model(self):
+        """Init model function in interpreter class.
+
+        Raises:
+            ValueError: Models should not be initialized within interpreter class, use load_model().
+        """
         raise ValueError("models should not be initialized within interpreter class, use load_model()")
 
     def load_model(
@@ -59,17 +65,17 @@ class InterpreterBase(estimators.Estimator):
         expected_pickle: Optional[list] = None,
         lateral_resolution: float = 1.0,
     ):
-        """
-        Load best or selected model from grid search directory.
+        """Load best or selected model from grid search directory.
 
-        :param results_path:
-        :param gs_id:
-        :param cv_idx:
-        :param subset_hyperparameters:
-        :param model_id:
-        :param expected_pickle:
-        :param lateral_resolution:
-        :return:
+        Args:
+            results_path (str): Path to results.
+            gs_id (str): Grid search identifier.
+            cv_idx (int): Cross-validation index.
+            subset_hyperparameters: Subset of hyperparameters.
+            model_id (str): Model identifier.
+            expected_pickle (list): List of expected pickle files, default "evaluation", "history", "hyperparam",
+                "model_args", "time"
+            lateral_resolution (float): Lateral resolution.
         """
         if subset_hyperparameters is None:
             subset_hyperparameters = []
@@ -111,20 +117,44 @@ class InterpreterBase(estimators.Estimator):
         prefetch: int = 100,
         reinit_n_eval: Optional[int] = None,
     ):
+        """Prepares a dataset.
+
+        Args:
+            image_keys (list): Image keys in partition.
+            nodes_idx (dict): Dictionary of nodes per image in partition.
+            batch_size (int): Batch size.
+            shuffle_buffer_size (int): Shuffle buffer size.
+            train (bool): Whether dataset is used for training or not (influences shuffling of nodes).
+            seed (int): Random seed.
+            prefetch (int): Prefetch of dataset.
+            reinit_n_eval (int): Used if model is reinitialized to different number of nodes per graph.
+        """
         pass
 
     def _get_resampled_dataset(
         self, image_keys: np.ndarray, nodes_idx: dict, batch_size: int, seed: Optional[int] = None, prefetch: int = 100
     ):
+        """Evaluates model based on resampled dataset for posterior resampling.
+        node_1 + domain_1 -> encoder -> z_1 + domain_2 -> decoder -> reconstruction_2
+
+        Args:
+            image_keys (list): Image keys in partition.
+            nodes_idx (dict): Dictionary of nodes per image in partition.
+            batch_size (int): Batch size.
+            seed (int): Random seed.
+            prefetch (int): Prefetch.
+        """
         pass
 
     def get_data_again(self, data_path: str, data_origin: str):
-        """
-        Loads data as previously done during model training.
+        """Loads data as previously done during model training.
 
-        :param data_path:
-        :param data_origin:
-        :return:
+        Args:
+            data_path (str): Data path.
+            data_origin (str): Data origin.
+
+        Raises:
+            ValueError: If `cond_type` is not recognized.
         """
         self.cond_type = self.gscontainer_runparams["cond_type"] if "cond_type" in self.gscontainer_runparams else None
         if self.cond_type == "gcn":
@@ -158,11 +188,10 @@ class InterpreterBase(estimators.Estimator):
         self.cell_names = list(self.node_type_names.values())
 
     def split_data_byidx_again(self, cv_idx: int):
-        """
-        Split data into partitions as done during model training.
+        """Split data into partitions as done during model training.
 
-        :param cv_idx: Index of cross-validation to plot confusion matrix for.
-        :return:
+        Args:
+            cv_idx (int): Index of cross-validation to plot confusion matrix for.
         """
         cv = self.gscontainer.select_cv(cv_idx=cv_idx)
         fn = f"{self.results_path}{self.gs_id}/results/{self.model_id}_{cv}_indices.pickle"
@@ -179,6 +208,11 @@ class InterpreterBase(estimators.Estimator):
         )
 
     def init_model_again(self):
+        """Initialize model in interpreter class.
+
+        Raises:
+            ValueError: If model_class not recognized
+        """
         if self.model_class in ["vae"]:
             model = models.ModelCVAE(**self._model_kwargs)
         elif self.model_class in ["cvae", "cvae_ncem"]:
@@ -200,9 +234,19 @@ class InterpreterBase(estimators.Estimator):
             self.vi_model = True
 
     def load_weights_again(self):
+        """Load model weights again in interpreter class."""
         self.model.training_model.load_weights(self._fn_model_weights)
 
     def reinitialize_model(self, changed_model_kwargs: dict, print_summary: bool = False):
+        """Reinitialize model with changed model kwargs.
+
+        Args:
+            changed_model_kwargs (dict): Dictionary over changed model kwargs.
+            print_summary (bool): Whether to print summary.
+
+        Raises:
+            ValueError: If model_class not recognized
+        """
         assert self.model is not None, "no model loaded, run init_model_again() first"
         # updating new model kwargs
         new_model_kwargs = self._model_kwargs.copy()
@@ -233,6 +277,18 @@ class InterpreterBase(estimators.Estimator):
     def _pp_saliencies(
         self, gradients, h_0, h_0_full, remove_own_gradient: bool = True, absolute_saliencies: bool = True
     ):
+        """Preprocessing of saliencies.
+
+        Args:
+            gradients (np.array): Array of gradients.
+            h_0 (np.array): h_0.
+            h_0_full (np.array): h_0_full.
+            remove_own_gradient (bool): Whether to remove own gradient.
+            absolute_saliencies (bool): Whether absolute saliencies should be aggregated.
+
+        Returns:
+            (np.array): Preprocessed saliencies.
+        """
         if self.cond_type == "max":
             gradients = np.nan_to_num(gradients)
         # removing gradient self node type
@@ -250,6 +306,16 @@ class InterpreterBase(estimators.Estimator):
         return sal
 
     def _neighbourhood_frequencies(self, a, h_0_full, discretize_adjacency: bool = True):
+        """Computes neighbourhood frequencies.
+
+        Args:
+            a (list):  List of adjacency matrices.
+            h_0_full (list): List of h_0_full matrices.
+            discretize_adjacency (bool): Whether to discretize the adjacency matrices.
+
+        Returns:
+            (np.array): Neighbourhood frequencies.
+        """
         neighbourhood = []
         for i, adj in enumerate(a):
             if discretize_adjacency:
@@ -260,20 +326,23 @@ class InterpreterBase(estimators.Estimator):
 
 
 class InterpreterLinear(estimators.EstimatorLinear, InterpreterBase):
-    """
-    Inherits all relevant functions specific to EstimatorLinear estimators
-    """
+    """Inherits all relevant functions specific to EstimatorLinear estimators."""
 
     def __init__(self):
+        """Initialize InterpreterLinear."""
         super().__init__()
 
     def _get_np_data(
         self, image_keys: Union[np.ndarray, str], nodes_idx: Union[dict, str]
     ) -> Tuple[Tuple[list, list, list, list, list], list]:
-        """
-        :param image_keys: Observation images indices.
-        :param nodes_idx: Observation nodes indices.
-        :return: Tuple of list of raw data, one list entry per image / graph
+        """Collects numpy objects from tensorflow dataset.
+
+        Args:
+            image_keys: Observation images indices.
+            nodes_idx: Observation nodes indices.
+
+        Returns:
+            Tuple of list of raw data, one list entry per image / graph
         """
         if isinstance(image_keys, (int, np.int32, np.int64)):
             image_keys = [image_keys]
@@ -288,7 +357,7 @@ class InterpreterLinear(estimators.EstimatorLinear, InterpreterBase):
         g = []
         h_obs = []
 
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             target_batch, source_batch, sf_batch, node_covar_batch, g_batch = x_batch
             target.append(target_batch.numpy().squeeze())
             source.append(source_batch.numpy().squeeze())
@@ -300,20 +369,23 @@ class InterpreterLinear(estimators.EstimatorLinear, InterpreterBase):
 
 
 class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
-    """
-    Inherits all relevant functions specific to EstimatorInteractions estimators
-    """
+    """Inherits all relevant functions specific to EstimatorInteractions estimators."""
 
     def __init__(self):
+        """Initialize InterepreterInteraction."""
         super().__init__()
 
     def _get_np_data(
         self, image_keys: Union[np.ndarray, str], nodes_idx: Union[dict, str]
     ) -> Tuple[Tuple[list, List[csr_matrix], list, list, list], list]:
-        """
-        :param image_keys: Observation images indices.
-        :param nodes_idx: Observation nodes indices.
-        :return: Tuple of list of raw data, one list entry per image / graph
+        """Collects numpy objects from tensorflow dataset.
+
+        Args:
+            image_keys: Observation images indices.
+            nodes_idx: Observation nodes indices.
+
+        Returns:
+            Tuple of list of raw data, one list entry per image / graph
         """
         if isinstance(image_keys, (int, np.int32, np.int64)):
             image_keys = [image_keys]
@@ -328,7 +400,7 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         g = []
         h_obs = []
 
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             target_batch, interaction_batch, sf_batch, node_covar_batch, g_batch = x_batch
             target.append(target_batch.numpy().squeeze())
             interactions.append(
@@ -352,9 +424,23 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         baseline_model,
         target_cell_type: str,
         undefined_type: Optional[str] = None,
-        n_neighbors: Optional[int] = None,
+        n_neighbors: int = 15,
         n_pcs: Optional[int] = None,
     ):
+        """Compute relative performance of spatial model compared to baseline model and subset to target cell type.
+
+        Args:
+            image_key (str): Image key.
+            baseline_model: Non-spatial baseline model.
+            target_cell_type (str): Target cell type.
+            undefined_type (str): Undefined type.
+            n_neighbors (int): The size of local neighborhood (in terms of number of neighboring data points) used
+                for manifold approximation.
+            n_pcs (int): Use this many PCs.
+
+        Returns:
+            (Tuple): AnnData object of image and AnnData object subsetted to only target cells.
+        """
         cluster_col = self.data.celldata.uns["metadata"]["cluster_col_preprocessed"]
         if isinstance(image_key, str):
             image_key = [image_key]
@@ -376,7 +462,7 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
                 )
                 graph = []
                 baseline = []
-                for step, (x_batch, y_batch) in enumerate(ds):
+                for _step, (x_batch, y_batch) in enumerate(ds):
                     out_graph = self.reinit_model.training_model(x_batch)
                     out_graph = np.split(ary=out_graph.numpy().squeeze(), indices_or_sections=2, axis=-1)[0]
 
@@ -422,6 +508,23 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         show: bool = True,
         return_axs: bool = False,
     ):
+        """Plots the relative the substate performance of a target cell type.
+
+        Args:
+            adata (AnnData): AnnData object. Output of `target_cell_relative_performance`.
+            target_cell_type (str): Target cell type.
+            relative_performance_key (str): Key for relative performance.
+            fontsize (int): Fontsize.
+            figsize (Tuple): Figure size.
+            palette (list): Palette.
+            save (str): Path to save directory.
+            suffix (str): Saving suffix.
+            show (bool): Whether to show figure.
+            return_axs (bool): Whether to return axes.
+
+        Returns:
+            Optionally returns axes or nothing.
+        """
         if palette is None:
             palette = ["#1f77b4", "#2ca02c", "#8c564b", "#7f7f7f", "#17becf"]
         if fontsize:
@@ -470,6 +573,25 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         show: bool = True,
         return_axs: bool = False,
     ):
+        """Plots relative performance of spatial versus non-spatial model on spatial allocation of nodes.
+
+        Args:
+            adata (AnnData): AnnData object. Output of `target_cell_relative_performance`.
+            target_cell_type (str): Target cell type.
+            relative_performance_key (str): Key for relative performance.
+            figsize (Tuple): Figure size.
+            fontsize (int): Fontsize.
+            spot_size (int): Spot size.
+            clean_view (bool): Whether to remove cells outside of range from plotting.
+            save (str): Path to save directory.
+            suffix (str): Saving suffix.
+            show (bool): Whether to show figure.
+            return_axs (bool): Whether to return axes.
+
+        Returns:
+            Optionally returns axes or nothing.
+        """
+
         class MidpointNormalize(colors.Normalize):
             def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
                 self.midpoint = midpoint
@@ -534,6 +656,24 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         suffix: str = "_expression_grid.pdf",
         show: bool = True,
     ):
+        """Plots a cell-cell grid of relative performance in terms of R2 or MAE.
+
+        Args:
+            image_keys (Union[np.array, str]): Image keys.
+            nodes_idx (Union[dict, str]): Node indices.
+            base_interpreter: Baseline model interpreter.
+            scale_node_frequencies (int): Value by which to scale the dots in the plot with.
+            metric (str): R2 or MAE.
+            mode (str): Mean or variance.
+            figsize (Tuple): figsize.
+            save (str): Path to save directory.
+            suffix (str): Saving suffix.
+            show (bool): Whether to show figure.
+
+        Raises:
+            ValueError: If metric or mode not recognized.
+        """
+
         class MidpointNormalize(colors.Normalize):
             def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
                 self.midpoint = midpoint
@@ -557,7 +697,7 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
 
         base_pred = []
         graph_pred = []
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             target_batch, interactions_batch, sf_batch, node_covar_batch, g_batch = x_batch
 
             base_pred.append(np.squeeze(base_interpreter.model.training_model(x_batch)[0].numpy()))
@@ -582,7 +722,7 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         plt.ioff()
         # function returns a n_features_type x n_features_type grid
         grid_summary = []
-        for i, k in enumerate(interaction_names):
+        for k in interaction_names:
             temp_interactions = interactions[interactions[k] > 0][k]
 
             if mode == "mean":
@@ -657,6 +797,16 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
         nodes_idx,
         significance_threshold: float = 0.01,
     ):
+        """Compute interaction parameter significance.
+
+        Args:
+            image_keys: Image keys.
+            nodes_idx: Node indices.
+            significance_threshold (float): Significance threshold for p-values.
+
+        Returns:
+            (Tuple): Interaction parameters of model, aaray of boolean significance, significance values.
+        """
         ds = self._get_dataset(
             image_keys=image_keys, nodes_idx=nodes_idx, batch_size=1, shuffle_buffer_size=1, train=False, seed=None
         )
@@ -674,7 +824,7 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
 
         interactions = []
         y = []
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             target_batch, interactions_batch, sf_batch, node_covar_batch, g_batch = x_batch
             interactions.append(
                 sparse.csr_matrix(
@@ -710,20 +860,23 @@ class InterpreterInteraction(estimators.EstimatorInteractions, InterpreterBase):
 
 
 class InterpreterGraph(estimators.EstimatorGraph, InterpreterBase):
-    """
-    Inherits all relevant functions specific to EstimatorGraph estimators
-    """
+    """Inherits all relevant functions specific to EstimatorGraph estimators."""
 
     def __init__(self):
+        """Initialize InterpreterGraph."""
         super().__init__()
 
     def _get_np_data(
         self, image_keys: Union[np.ndarray, str], nodes_idx: Union[dict, str]
     ) -> Tuple[Tuple[list, list, list, list, List[csr_matrix], List[csr_matrix], list, list], list]:
-        """
-        :param image_keys: Observation images indices.
-        :param nodes_idx: Observation nodes indices.
-        :return: Tuple of list of raw data, one list entry per image / graph
+        """Collects numpy objects from tensorflow dataset.
+
+        Args:
+            image_keys: Observation images indices.
+            nodes_idx: Observation nodes indices.
+
+        Returns:
+            Tuple of list of raw data, one list entry per image / graph
         """
         if isinstance(image_keys, (int, np.int32, np.int64)):
             image_keys = [image_keys]
@@ -741,7 +894,7 @@ class InterpreterGraph(estimators.EstimatorGraph, InterpreterBase):
         g = []
         h_obs = []
 
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             h_1_batch, sf_batch, h_0_batch, h_0_full_batch, a_batch, a_full_batch, node_covar_batch, g_batch = x_batch
             h_1.append(h_1_batch.numpy().squeeze())
             sf.append(sf_batch.numpy().squeeze())
@@ -769,11 +922,10 @@ class InterpreterGraph(estimators.EstimatorGraph, InterpreterBase):
 
 
 class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
-    """
-    Inherits all relevant functions specific to EstimatorInteractions estimators
-    """
+    """Inherits all relevant functions specific to EstimatorInteractions estimators."""
 
     def __init__(self):
+        """Initialize InterepreterEDncem."""
         super().__init__()
 
     def target_cell_saliencies(
@@ -784,6 +936,17 @@ class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
         partition: str = "test",
         multicolumns: Optional[list] = None,
     ):
+        """Compute target cell saliencies.
+
+        Args:
+            target_cell_type (str): Target cell type.
+            drop_columns: List of columns to drop, i.e. neighbouring cell types.
+            dop_images: List of images to drop.
+            partition (str): All, train, test or val partition. Default is test.
+            multicolumns: List of items in multiindex annotation.
+        Returns:
+            Dataframe of target cell saliencies in partition.
+        """
         target_cell_idx = list(self.node_type_names.values()).index(target_cell_type)
 
         if partition == "test":
@@ -823,7 +986,7 @@ class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
                 h_0 = []
                 h_0_full = []
                 a = []
-                for step, (x_batch, y_batch) in enumerate(ds):
+                for _step, (x_batch, _y_batch) in enumerate(ds):
                     h_1_batch = x_batch[0].numpy().squeeze()
                     h_0_batch = x_batch[2].numpy().squeeze()  # 1 x 1 x node_types
                     h_0_full_batch = x_batch[3]  # 1 x max_nodes x node_types
@@ -897,6 +1060,19 @@ class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
         show: bool = True,
         return_axs: bool = False,
     ):
+        """Plot target cell saliencies from dataframe.
+
+        Args:
+            saliencies: Precomputed saliencies from 'target_cell_saliencies'.
+            multiindex (bool): Whether saliencies have multiindex.
+            fontsize (int): Fontsize.
+            figsize (Tuple): Figsize.
+            width_ratios (list): Width ratios.
+            save (str): Path to save directory.
+            suffix (str): Saving suffix.
+            show (bool): Whether to show figure.
+            return_axs (bool): Whether to return axes.
+        """
         if width_ratios is None:
             width_ratios = [5, 1]
         if fontsize:
@@ -915,7 +1091,7 @@ class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
 
             hline = []
             new_xlabels = []
-            for index1, index2_list in xlabel_mapping.items():
+            for _index1, index2_list in xlabel_mapping.items():
                 index2_list[0] = "{}".format(index2_list[0])
                 new_xlabels.extend(index2_list)
 
@@ -940,23 +1116,35 @@ class InterpreterEDncem(estimators.EstimatorEDncem, InterpreterGraph):
 
 
 class InterpreterCVAEncem(estimators.EstimatorCVAEncem, InterpreterGraph):
-    """
-    Inherits all relevant functions specific to EstimatorInteractions estimators
-    """
+    """Inherits all relevant functions specific to EstimatorInteractions estimators."""
 
     def __init__(self):
+        """Initialize InterepreterCVAEncem."""
         super().__init__()
 
     def compute_latent_space_cluster_enrichment(
         self,
         image_key,
         target_cell_type: str,
-        n_neighbors: Optional[int] = None,
+        n_neighbors: int = 15,
         n_pcs: Optional[int] = None,
         filter_titles: Optional[List[str]] = None,
         clip_pvalues: Optional[int] = -5,
     ):
+        """Compute latent space cluster enrichment.
 
+        Args:
+            image_key: Image key.
+            target_cell_type (str): Target cell type.
+            n_neighbors (int): The size of local neighborhood (in terms of number of neighboring data points) used
+                for manifold approximation.
+            n_pcs (int): Use this many PCs.
+            filter_titles (list): Filter titles.
+            clip_pvalues (int): Clip p-values by values.
+
+        Returns:
+            (Tuple): Copy of latent space AnnData object, dataframe of log10 p-values, dataframe of fold_change
+        """
         node_type_names = list(self.data.celldata.uns["node_type_names"].values())
         ds = self._get_dataset(
             image_keys=[image_key],
@@ -975,7 +1163,7 @@ class InterpreterCVAEncem(estimators.EstimatorCVAEncem, InterpreterGraph):
         h_0_full = []
         h_0 = []
         a = []
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, _y_batch) in enumerate(ds):
             h_1_batch, sf_batch, h_0_batch, h_0_full_batch, a_batch, a_full_batch, node_covar_batch, g_batch = x_batch
 
             cond_z, cond_z_mean, cond_z_log = cond_encoder(x_batch)
@@ -1006,7 +1194,7 @@ class InterpreterCVAEncem(estimators.EstimatorCVAEncem, InterpreterGraph):
         sc.tl.louvain(cond_adata)
         sc.tl.umap(cond_adata)
 
-        for i, x in enumerate(node_type_names):
+        for x in node_type_names:
             cond_adata.uns[f"{x}_colors"] = ["darkgreen", "lightgrey"]
 
         cond_adata.obs["substates"] = target_cell_type + " " + cond_adata.obs.louvain.astype(str)
@@ -1021,7 +1209,7 @@ class InterpreterCVAEncem(estimators.EstimatorCVAEncem, InterpreterGraph):
 
         distinct_louvain = len(np.unique(cond_adata.obs.louvain))
         pval_source_type = []
-        for i, st in enumerate(node_type_names):
+        for st in node_type_names:
             pval_cluster = []
             for j in range(distinct_louvain):
                 crosstab = np.array(pd.crosstab(df[f"{st}"], df[str(j)]))
@@ -1058,20 +1246,23 @@ class InterpreterCVAEncem(estimators.EstimatorCVAEncem, InterpreterGraph):
 
 
 class InterpreterNoGraph(estimators.EstimatorNoGraph, InterpreterBase):
-    """
-    Inherits all relevant functions specific to EstimatorEDncem estimators and InterpreterBase
-    """
+    """Inherits all relevant functions specific to EstimatorEDncem estimators and InterpreterBase."""
 
     def __init__(self):
+        """Initialize InterepreterNoGraph.."""
         super().__init__()
 
     def _get_np_data(
         self, image_keys: Union[np.ndarray, str], nodes_idx: Union[dict, str]
     ) -> Tuple[Tuple[list, list, list, list], list]:
-        """
-        :param image_keys: Observation images indices.
-        :param nodes_idx: Observation nodes indices.
-        :return: Tuple of list of raw data, one list entry per image / graph
+        """Collects numpy objects from tensorflow dataset.
+
+        Args:
+            image_keys: Observation images indices.
+            nodes_idx: Observation nodes indices.
+
+        Returns:
+            Tuple of list of raw data, one list entry per image / graph
         """
         if isinstance(image_keys, (int, np.int32, np.int64)):
             image_keys = [image_keys]
@@ -1085,7 +1276,7 @@ class InterpreterNoGraph(estimators.EstimatorNoGraph, InterpreterBase):
         g = []
         h_obs = []
 
-        for step, (x_batch, y_batch) in enumerate(ds):
+        for _step, (x_batch, y_batch) in enumerate(ds):
             h_1_batch, sf_batch, node_covar_batch, g_batch = x_batch
             h_1.append(h_1_batch.numpy().squeeze())
             sf.append(sf_batch.numpy().squeeze())
