@@ -1,4 +1,5 @@
+"""Importing GridSearchContrainer and TrainModel classes."""
 from ncem.train.summaries import GridSearchContainer
 from ncem.train.train_model import (TrainModelCVAE, TrainModelCVAEncem,
-                                    TrainModelED, TrainModelEDncem,
+                                    TrainModelED, TrainModelEDncem, TrainModelEdSingleNcem,
                                     TrainModelInteractions, TrainModelLinear)
