@@ -11,11 +11,10 @@ from matplotlib.ticker import FormatStrFormatter
 
 
 class GridSearchContainer:
+    """GridSearchContainer class."""
+
     runparams: dict
     run_ids_clean: dict
-    evals: dict
-    evals_posterior_sampling: dict
-    histories: dict
     source_gs: dict
     cv_ids: dict
 
@@ -23,21 +22,28 @@ class GridSearchContainer:
     target_cell_evals: dict
     target_cell_indices: dict
 
-    keys_all: list
-    keys_test: list
-    keys_train: list
-    keys_val: list
-    nodes_test: Dict[str, np.ndarray]
-    nodes_train: Dict[str, np.ndarray]
-    nodes_val: Dict[str, np.ndarray]
-
     def __init__(self, source_path, gs_ids, lateral_resolution):
+        """Initialize GridSearchContainer.
+
+        Parameters
+        ----------
+        source_path
+            Source path.
+        gs_ids
+            Grid search identifiers.
+        lateral_resolution
+            Lateral resolution.
+        """
         self.source_path = source_path
         if isinstance(gs_ids, str):
             gs_ids = [gs_ids]
         self.gs_ids = gs_ids
         self.lateral_resolution = lateral_resolution
 
+        self.evals = {}
+        self.evals_posterior_sampling = {}
+        self.histories = {}
+
         self.summary_table = None
         self.runparams_table = None
         self.target_cell_table = None
@@ -46,10 +52,11 @@ class GridSearchContainer:
 
     @property
     def cv_keys(self) -> List[str]:
-        """
-        Returns keys of cross-validation used in dictionaries in this class.
+        """Return keys of cross-validation used in dictionaries in this class.
 
-        :return: list of string keys
+        Returns
+        -------
+        list of string keys
         """
         return np.unique(self.summary_table["cv"].values).tolist()
 
@@ -59,12 +66,23 @@ class GridSearchContainer:
         add_posterior_sampling_model: bool = False,
         report_unsuccessful_runs: bool = False,
     ):
-        """
-        Load all metrics from grid search output files.
+        """Load all metrics from grid search output files.
 
         Core results are save in self.summary_table.
 
-        :return:
+        Parameters
+        ----------
+        expected_pickle : list, optional
+            Expected pickle files.
+        add_posterior_sampling_model : bool
+            Whether to add posterior sampling model as seperate model to summary_table.
+        report_unsuccessful_runs : bool
+            Whether to print reporting statements in out stream.
+
+        Raises
+        ------
+        ValueError
+            If no complete runs found.
         """
         if expected_pickle is None:
             expected_pickle = ["evaluation", "history", "hyperparam", "model_args", "time"]
@@ -101,7 +119,7 @@ class GridSearchContainer:
                 np.unique(["_".join(x.split("_")[:-1]) for x in run_ids])  # identifiers of hyper-parameters settings
             )
             run_ids_clean = []  # only IDs of completed runs (all files present)
-            for j, r in enumerate(run_ids):
+            for r in run_ids:
                 complete_run = True
                 for cv in cv_ids:
                     # Check pickled files:
@@ -163,12 +181,13 @@ class GridSearchContainer:
                     else "none",
                     "gs_id": [runparams[x]["gs_id"] for x in run_ids_clean],
                     "model_id": [runparams[x]["model_id"] for x in run_ids_clean],
-                    "split_mode": [runparams[x]["split_mode"] for x in run_ids_clean],
-                    "radius": [runparams[x]["max_dist"] for x in run_ids_clean],
+                    #"split_mode": [runparams[x]["split_mode"] for x in run_ids_clean],
+                    "radius": [runparams[x]["radius"] for x in run_ids_clean],
+                    "n_rings": [runparams[x]["n_rings"] for x in run_ids_clean] if "n_rings" in list(runparams[x].keys()) else "none",
                     "graph_covar_selection": [runparams[x]["graph_covar_selection"] for x in run_ids_clean],
-                    "node_label_space_id": [runparams[x]["node_feature_space_id_0"] for x in run_ids_clean],
-                    "node_feature_space_id": [runparams[x]["node_feature_space_id_1"] for x in run_ids_clean],
-                    "feature_transformation": [runparams[x]["feature_transformation"] for x in run_ids_clean],
+                    "node_label_space_id": [runparams[x]["node_label_space_id"] for x in run_ids_clean],
+                    "node_feature_space_id": [runparams[x]["node_feature_space_id"] for x in run_ids_clean],
+                    #"feature_transformation": [runparams[x]["feature_transformation"] for x in run_ids_clean],
                     "use_covar_node_position": [runparams[x]["use_covar_node_position"] for x in run_ids_clean],
                     "use_covar_node_label": [runparams[x]["use_covar_node_label"] for x in run_ids_clean],
                     "use_covar_graph_covar": [runparams[x]["use_covar_graph_covar"] for x in run_ids_clean],
@@ -227,12 +246,14 @@ class GridSearchContainer:
                     else "none",
                     "use_domain": [runparams[x]["use_domain"] for x in run_ids_clean],
                     "domain_type": [runparams[x]["domain_type"] for x in run_ids_clean],
-                    "use_batch_norm": [runparams[x]["use_batch_norm"] for x in run_ids_clean],
+                    "use_batch_norm": [runparams[x]["use_batch_norm"] for x in run_ids_clean] if "use_batch_norm" in list(runparams[x].keys())
+                    else "none",
                     "use_type_cond": [runparams[x]["use_type_cond"] for x in run_ids_clean]
                     if "use_type_cond" in list(runparams[x].keys())
                     else "none",
                     "scale_node_size": [runparams[x]["scale_node_size"] for x in run_ids_clean],
-                    "transform_input": [runparams[x]["transform_input"] for x in run_ids_clean],
+                    "transform_input": [runparams[x]["transform_input"] for x in run_ids_clean] if "transform_input" in list(runparams[x].keys())
+                    else "none",
                     "output_layer": [runparams[x]["output_layer"] for x in run_ids_clean],
                     "log_transform": [runparams[x]["log_transform"] for x in run_ids_clean]
                     if "log_transform" in list(runparams[x].keys())
@@ -420,33 +441,16 @@ class GridSearchContainer:
         self.runparams_table = pd.concat(self.runparams_table)
         self.summary_table["um_radius"] = (self.summary_table["radius"] * self.lateral_resolution).astype(int)
 
-        # Load files that are shared across a grid search.
-        keys_all = {}
-        keys_test = {}
-        keys_train = {}
-        keys_val = {}
-        nodes_test = {}
-        nodes_train = {}
-        nodes_val = {}
-        for cv in cv_ids:
-            with open(indir + run_ids_clean[0] + "_" + cv + "_indices.pickle", "rb") as f:
-                idx_partitions = pickle.load(f)
-            keys_all[cv] = idx_partitions["all"]
-            keys_test[cv] = idx_partitions["test"]
-            keys_train[cv] = idx_partitions["train"]
-            keys_val[cv] = idx_partitions["val"]
-            nodes_test[cv] = idx_partitions["test_nodes"]
-            nodes_train[cv] = idx_partitions["train_nodes"]
-            nodes_val[cv] = idx_partitions["val_nodes"]
-        self.keys_all = keys_all
-        self.keys_test = keys_test
-        self.keys_train = keys_train
-        self.keys_val = keys_val
-        self.nodes_test = nodes_test
-        self.nodes_train = nodes_train
-        self.nodes_val = nodes_val
-
     def load_target_cell_evaluation(self, report_unsuccessful_runs: bool = False):
+        """Load all metrics from grid search output files of target cell evaluation.
+
+        Core results are save in self.target_cell_table.
+
+        Parameters
+        ----------
+        report_unsuccessful_runs : bool
+            Whether to print reporting statements in out stream.
+        """
         self.target_cell_runparams = {}
         self.target_cell_evals = {}
         self.target_cell_indices = {}
@@ -471,7 +475,7 @@ class GridSearchContainer:
                 np.unique(["_".join(x.split("_")[:-1]) for x in run_ids])  # identifiers of hyper-parameters settings
             )
             run_ids_clean = []  # only IDs of completed runs (all files present)
-            for j, r in enumerate(run_ids):
+            for r in run_ids:
                 complete_run = True
                 for cv in cv_ids:
                     fn = r + "_" + cv + "_ntevaluation.pickle"
@@ -537,20 +541,21 @@ class GridSearchContainer:
                                                         else "none",
                                                         "gs_id": [runparams[x]["gs_id"]],
                                                         "model_id": [runparams[x]["model_id"]],
-                                                        "split_mode": [runparams[x]["split_mode"]],
-                                                        "radius": [runparams[x]["max_dist"]],
+                                                        #"split_mode": [runparams[x]["split_mode"]],
+                                                        "radius": [runparams[x]["radius"]],
+                                                        "n_rings": [runparams[x]["n_rings"]] if "n_rings" in list(runparams[x].keys()) else "none",
                                                         "graph_covar_selection": [
                                                             runparams[x]["graph_covar_selection"]
                                                         ],
                                                         "node_label_space_id": [
-                                                            runparams[x]["node_feature_space_id_0"]
+                                                            runparams[x]["node_label_space_id"]
                                                         ],
                                                         "node_feature_space_id": [
-                                                            runparams[x]["node_feature_space_id_1"]
-                                                        ],
-                                                        "feature_transformation": [
-                                                            runparams[x]["feature_transformation"]
+                                                            runparams[x]["node_feature_space_id"]
                                                         ],
+                                                        #"feature_transformation": [
+                                                        #    runparams[x]["feature_transformation"]
+                                                        #],
                                                         "use_covar_node_position": [
                                                             runparams[x]["use_covar_node_position"]
                                                         ],
@@ -561,19 +566,19 @@ class GridSearchContainer:
                                                         # "hold_out_covariate": [runparams[x]['hold_out_covariate']],
                                                         "optimizer": [runparams[x]["optimizer"]],
                                                         "learning_rate": [runparams[x]["learning_rate"]],
-                                                        "intermediate_dim_enc": [runparams[x]["intermediate_dim_enc"]],
-                                                        "intermediate_dim_dec": [runparams[x]["intermediate_dim_dec"]],
-                                                        "latent_dim": [runparams[x]["latent_dim"]],
+                                                        #"intermediate_dim_enc": [runparams[x]["intermediate_dim_enc"]],
+                                                        #"intermediate_dim_dec": [runparams[x]["intermediate_dim_dec"]],
+                                                        #"latent_dim": [runparams[x]["latent_dim"]],
                                                         # "depth_enc": [runparams[x]['depth_enc']],
                                                         # "depth_dec": [runparams[x]['depth_dec']],
-                                                        "dropout_rate": [runparams[x]["dropout_rate"]],
+                                                        #"dropout_rate": [runparams[x]["dropout_rate"]],
                                                         "l2_coef": [runparams[x]["l2_coef"]],
                                                         "l1_coef": [runparams[x]["l1_coef"]],
                                                         "use_domain": [runparams[x]["use_domain"]],
                                                         "domain_type": [runparams[x]["domain_type"]],
-                                                        "use_batch_norm": [runparams[x]["use_batch_norm"]],
+                                                        #"use_batch_norm": [runparams[x]["use_batch_norm"]],
                                                         "scale_node_size": [runparams[x]["scale_node_size"]],
-                                                        "transform_input": [runparams[x]["transform_input"]],
+                                                        #"transform_input": [runparams[x]["transform_input"]],
                                                         "output_layer": [runparams[x]["output_layer"]],
                                                         "log_transform": [runparams[x]["log_transform"]]
                                                         if "log_transform" in list(runparams[x].keys())
@@ -624,10 +629,21 @@ class GridSearchContainer:
         self.target_cell_table["um_radius"] = (self.target_cell_table["radius"] * self.lateral_resolution).astype(int)
 
     def select_cv(self, cv_idx: int) -> str:
-        """
-        Return key of of cross-validation selected with numeric index.
-        :param cv_idx: Index of cross-validation to plot confusion matrix for.
-        :return:
+        """Return key of of cross-validation selected with numeric index.
+
+        Parameters
+        ----------
+        cv_idx : int
+            Index of cross-validation to plot confusion matrix for.
+
+        Returns
+        -------
+        cv
+
+        Raises
+        ------
+        ValueError
+            `cv_idx` out of scope of cross-validation set
         """
         if cv_idx >= len(self.cv_keys):
             raise ValueError("cv_idx %i out of scope of cross-validation set: %s" % (cv_idx, self.cv_keys))
@@ -643,6 +659,30 @@ class GridSearchContainer:
         partition_select: str = "test",
         cv_mode: str = "mean",
     ):
+        """Get best model identifier.
+
+        Parameters
+        ----------
+        subset_hyperparameters : list, optional
+            List of subset hyperparameters.
+        metric_select : str
+            Selected metric.
+        partition_select : str
+            Selected parition.
+        cv_mode : str
+            cross validation mode.
+
+        Returns
+        -------
+        best_model_id
+
+        Raises
+        ------
+        ValueError
+            If measure, partition or cv_mode not recognized.
+        Warning
+            If cv_mode max is selected with the following metrics: loss, elbo, mse, mae
+        """
         if subset_hyperparameters is None:
             subset_hyperparameters = []
         if metric_select.endswith("loss"):
@@ -714,6 +754,21 @@ class GridSearchContainer:
         partition_select: str = "val",
         cv_mode: str = "mean",
     ):
+        """Copy best model.
+
+        Parameters
+        ----------
+        gs_id : str
+            Grid search identifier.
+        dst : str
+            dst folder.
+        metric_select : str
+            Selected metric.
+        partition_select : str
+            Selected partition.
+        cv_mode : str
+            Cross validation mode.
+        """
         from shutil import copyfile
 
         if dst[0] != "/":
@@ -728,7 +783,21 @@ class GridSearchContainer:
             fn_idx = run_id + "_" + cv + "_indices.pickle"
             copyfile(src + fn_idx, dst + fn_idx)
 
-    def get_info(self, model_id, expected_pickle: Union[None, list] = None):
+    def get_info(self, model_id, expected_pickle: Optional[list] = None):
+        """Get information of model.
+
+        Parameters
+        ----------
+        model_id
+            Model identifier.
+        expected_pickle : list, optional
+            Expected pickle files.
+
+        Raises
+        ------
+        ValueError
+            If file is missing.
+        """
         if expected_pickle is None:
             expected_pickle = ["evaluation", "history", "hyperparam"]
 
@@ -755,8 +824,7 @@ class GridSearchContainer:
             info["runparams"] = pickle.load(f)
         self.info = info
 
-        # Plotting functions:
-
+    # Plotting functions:
     def plot_best_model_by_hyperparam(
         self,
         partition_show: str,
@@ -776,7 +844,6 @@ class GridSearchContainer:
         figsize: Tuple[float, float] = (3.5, 4.0),
         fontsize: Optional[int] = None,
         plot_mode: str = "boxplot",
-        hue_order=None,
         palette: Optional[dict] = None,
         color: Optional[str] = None,
         save: Optional[str] = None,
@@ -784,6 +851,61 @@ class GridSearchContainer:
         show: bool = True,
         return_axs: bool = False,
     ):
+        """Plot best model by hyperparameter.
+
+        Parameters
+        ----------
+        partition_show : str
+            Showing partition.
+        metric_show : str
+            Showing metric.
+        partition_select : str
+            Selected partition.
+        metric_select : str
+            Selected metric.
+        param_x : str
+            Parameter on x axis.
+        param_hue : str
+            Parameter for hue.
+        graph_model_class : str
+            Graph model class.
+        baseline_model_class : str
+            Baseline model class.
+        rename_levels : list, optional
+            Rename levels with stated logic.
+        subset_hyperparameters : list, optional
+            Subset hyperparameters.
+        cv_mode : str
+            Cross validation mode.
+        yaxis_limit : tuple, optional
+            y axis limits.
+        xticks : list, optional
+            List of x ticks.
+        rotate_xticks : bool
+            Whether to rotate x ticks.
+        figsize : tuple
+            Figure size.
+        fontsize : int, optional
+            Font size.
+        plot_mode : str
+            Plotting mode, can be `boxplot`, `lineplot` or `mean_lineplot`.
+        palette : dict, optional
+            Palette.
+        color : str, optional
+            Color.
+        save : str, optional
+            Whether (if not None) and where (path as string given as save) to save plot.
+        suffix : str
+            Suffix of file name to save to.
+        show : bool
+            Whether to display plot.
+        return_axs : bool
+            Whether to return axis objects.
+
+        Returns
+        -------
+        axis if `return_axs` is True.
+        """
         if fontsize:
             sc.set_figure_params(scanpy=True, fontsize=fontsize)
         sns.set_palette("colorblind")
@@ -842,7 +964,6 @@ class GridSearchContainer:
                     x=param_x,
                     y=ycol,
                     hue=param_hue,
-                    hue_order=hue_order,
                     data=summary_table,
                     ax=ax,
                 )
@@ -862,7 +983,6 @@ class GridSearchContainer:
                 x=param_x,
                 y=ycol,
                 style="cv",
-                hue_order=hue_order,
                 palette=palette,
                 data=summary_table[summary_table["model_class"] == graph_model_class],
                 hue=param_hue,
@@ -925,14 +1045,39 @@ class GridSearchContainer:
         metric_select: str,
         param_x: str,
         ncols: int = 8,
-        plot_mode: str = "boxplot",
         show: bool = True,
         save: Optional[str] = None,
         suffix: str = "target_cell_evaluation.pdf",
         return_axs: bool = False,
-        panel_width: float = 3.0,
-        panel_height: float = 3.0,
+        panelsize: Tuple[float, float] = (3.0, 3.0),
     ):
+        """Plot target cell evaluation.
+
+        Parameters
+        ----------
+        metric_show : str
+            Showing metric.
+        metric_select : str
+            Selected metric.
+        param_x : str
+            Parameter on x axis.
+        panelsize : tuple
+            Panel size.
+        ncols : int
+            Number of columns.
+        save : str, optional
+            Whether (if not None) and where (path as string given as save) to save plot.
+        suffix : str
+            Suffix of file name to save to.
+        show : bool
+            Whether to display plot.
+        return_axs : bool
+            Whether to return axis objects.
+
+        Returns
+        -------
+        axis if `return_axs` is True.
+        """
         params_x_unique = np.sort(np.unique(self.target_cell_table[param_x].values))
         params_tc_unique = np.sort(np.unique(self.target_cell_table["target_cell"].values))
 
@@ -940,7 +1085,7 @@ class GridSearchContainer:
         nrows = len(params_tc_unique) // ncols + int(len(params_tc_unique) % ncols > 0)
 
         fig, ax = plt.subplots(
-            ncols=ncols, nrows=nrows, figsize=(ncols * panel_width, nrows * panel_height), sharex=True
+            ncols=ncols, nrows=nrows, figsize=(ncols * panelsize[0], nrows * panelsize[1]), sharex=True
         )
         ax = ax.flat
         for a in ax[ct:]:
@@ -977,11 +1122,8 @@ class GridSearchContainer:
             table = target_cell_table.loc[np.array([x in run_ids for x in target_cell_table["run_id"].values]), :]
             tc_frequencies = int(np.unique(table["target_cell_frequencies"])[0])
             target_cell_frequencies.update({tc: tc_frequencies})
-            if plot_mode == "boxplot":
-                sns.boxplot(x=param_x, y=metric_show, data=table, ax=ax[i], color="steelblue")
-                sns.swarmplot(x=param_x, y=metric_show, data=table, ax=ax[i], color="steelblue")
-            elif plot_mode == "lineplot":
-                sns.lineplot(x=param_x, y=metric_show, style="cv", data=table, ax=ax[i], sort=False, markers=True)
+            sns.boxplot(x=param_x, y=metric_show, data=table, ax=ax[i], color="steelblue")
+            sns.swarmplot(x=param_x, y=metric_show, data=table, ax=ax[i], color="steelblue")
             ax[i].set_title(f"{tc.replace('_', ' ')} \n({str(tc_frequencies)} cells)")
             ax[i].set_xlabel("")
             ax[i].set_ylabel("")
