@@ -2,9 +2,21 @@ import tensorflow as tf
 
 
 def sparse_dense_matmult_batch(sp_a, b):
-    """
-    Multiplies a tf.SparseTensor sp_a with an additional batch dimension with a 2 dimensional
-    dense matrix. Returns a 3 dimensional dense matrix.
+    """Multiply sparse with dense.
+
+    Multiplies a tf.SparseTensor sp_a with an additional batch dimension with a 2 dimensional dense matrix and
+    returns a 3 dimensional dense matrix.
+
+    Parameters
+    ----------
+    sp_a
+        Sparse a matrix.
+    b
+        Dense b matrix.
+
+    Returns
+    -------
+    dense_mat.
     """
 
     def map_function(x):
