@@ -1,10 +1,21 @@
 import numpy as np
 
 
-def _get_p_value(a_var: np.ndarray, fisher_inv: np.ndarray, coef_loc_totest: int):
-    """
-    Returns the p-value for differential expression for each gene
-    :return: np.ndarray
+def _get_p_value(a_var: np.array, fisher_inv: np.array, coef_loc_totest: int):
+    """Return the p-value for differential expression for each gene.
+
+    Parameters
+    ----------
+    a_var : np.array
+        a var matrix.
+    fisher_inv : np.array
+        Fisher inv matrix.
+    coef_loc_totest : int
+        Coefficient location to test.
+
+    Returns
+    -------
+    wald_test
     """
     from diffxpy.stats.stats import wald_test
 
@@ -16,10 +27,25 @@ def _get_p_value(a_var: np.ndarray, fisher_inv: np.ndarray, coef_loc_totest: int
 
 
 def wald_test(
-    parameters,
+    params,
     fisher_inv: list,
     significance_threshold: float = 0.01,
 ):
+    """Compute wald test.
+
+    Parameters
+    ----------
+    params
+        Parameters.
+    fisher_inv : list
+        List of fisher inv matrix.
+    significance_threshold : float
+        Significance threshold for corrected p-values.
+
+    Returns
+    -------
+    bool_res, res
+    """
     from diffxpy.testing.correction import correct
 
     res = []
@@ -27,7 +53,7 @@ def wald_test(
     for i, fi in enumerate(fisher_inv):
         sig = []
         bool_sig = []
-        a_var = parameters[:, i, :].T
+        a_var = params[:, i, :].T
         for idx in range(a_var.shape[0]):
             p_val = _get_p_value(a_var, fi, idx)
             significance = correct(p_val)
